package justfun.common.pool;

import java.util.EventListener;

import javax.swing.event.EventListenerList;

import justfun.common.pool.event.PoolActionEvent;
import justfun.common.pool.event.PoolActionListener;

/**
 * 이벤트 기반의 추상 Pool 구현
 * 
 * @author 조재호
 */
public abstract class EventDrivenPool implements Pool {

	protected EventListenerList listenerList = new EventListenerList();
	protected boolean ignoreEvent = false;

	public void setIgnoreEvent(boolean ignoreEvent) {
		this.ignoreEvent = ignoreEvent;
	}

	/**
	 * Pool로부터 Object를 얻는다.
	 * 
	 * @return
	 * @throws PoolException
	 */
	protected abstract Object doRequestObject() throws PoolException;

	public Object requestObject() throws PoolException {
		firePoolActionEventOnRequest(new PoolActionEvent(this));

		return doRequestObject();
	}

	/**
	 * Pool에 Object를 반환한다.
	 * 
	 * @param obj
	 * @throws PoolException
	 */
	protected abstract void doReturnObject(Object obj) throws PoolException;

	public void returnObject(Object obj) throws PoolException {
		firePoolActionEventOnReturn(new PoolActionEvent(this));

		doReturnObject(obj);

		firePoolActionEventAfterReturn(new PoolActionEvent(this));
	}

	/**
	 * 리스너 등록
	 * 
	 * @param listener
	 */
	public void addPoolActionListener(PoolActionListener listener) {
		listenerList.add(PoolActionListener.class, listener);
	}

	/**
	 * 리스너 제거
	 * 
	 * @param listener
	 */
	public void removePoolActionListener(PoolActionListener listener) {
		listenerList.remove(PoolActionListener.class, listener);
	}

	/**
	 * 리스너 반환
	 * 
	 * @return
	 */
	public PoolActionListener[] getPoolActionListeners() {
		return (PoolActionListener[]) getListeners(PoolActionListener.class);
	}

	/**
	 * 리스너 반환
	 * 
	 * @param clazz
	 * @return
	 */
	public EventListener[] getListeners(Class clazz) {
		return listenerList.getListeners(clazz);
	}

	/**
	 * 이벤트를 발생시키고 모든 리스너에게 공지
	 * 
	 * @param event
	 */
	protected void firePoolActionEventOnRequest(PoolActionEvent event) {
		if (!this.ignoreEvent) {
			Object[] list = listenerList.getListenerList();
			for (int i = 0; i < list.length; i += 2) {
				Class listenerClass = (Class) list[i];
				if (listenerClass == PoolActionListener.class) {
					PoolActionListener listener = (PoolActionListener) list[i + 1];
					listener.onRequestObject(event);
				}
			}
		}
	}

	protected void firePoolActionEventOnReturn(PoolActionEvent event) {
		if (!this.ignoreEvent) {
			Object[] list = listenerList.getListenerList();
			for (int i = 0; i < list.length; i += 2) {
				Class listenerClass = (Class) list[i];
				if (listenerClass == PoolActionListener.class) {
					PoolActionListener listener = (PoolActionListener) list[i + 1];
					listener.onReturnObject(event);
				}
			}
		}
	}

	protected void firePoolActionEventAfterReturn(PoolActionEvent event) {
		if (!this.ignoreEvent) {
			Object[] list = listenerList.getListenerList();
			for (int i = 0; i < list.length; i += 2) {
				Class listenerClass = (Class) list[i];
				if (listenerClass == PoolActionListener.class) {
					PoolActionListener listener = (PoolActionListener) list[i + 1];
					listener.afterReturnObject(event);
				}
			}
		}
	}

}
