package justfun.common.cache;

import java.util.Set;

/**
 * 캐쉬 인터페이스 정의
 * 
 * @author 조재호
 */
public interface Cache {

	public static final long UNLIMITED = -1;

	/**
	 * 캐쉬에 데이터를 넣는다.
	 * 
	 * @param key -
	 *            해쉬테이블 키
	 * @param value -
	 *            캐쉬에 넣을 데이터
	 * @param expireTime -
	 *            데이터 만료일
	 * @return 캐쉬에 넣은 데이터
	 */
	public Object put(Object key, Object value, long expireTime);

	/**
	 * 캐쉬로부터 데이터를 얻는다.
	 * 
	 * @param key
	 * @return 캐쉬 데이터 - 해당 key의 캐쉬 데이터가 존재하지 않으면, <code>null</code>을 리턴한다. 만약
	 *         데이터가 존재하더라도 만료일이 지났다면 캐쉬에서 데이터가 제거되고 <code>null</code>을 리턴한다.
	 */
	public Object get(Object key);

	/**
	 * 캐쉬 데이터의 생성일을 구한다.
	 * 
	 * @param key
	 * @return 해당 캐쉬 데이터의 생성시간. 해당 데이터가 없으면 0을 리턴
	 */
	public long getCreatedTime(Object key);

	/**
	 * 캐쉬 데이터의 최근 사용일을 구한다.
	 * 
	 * @param key
	 * @return 해당 캐쉬 데이터의 최근 사용시간. 해당 데이터가 없으면 0을 리턴
	 */
	public long getLastAccessTime(Object key);

	/**
	 * 캐쉬 데이터의 만료일을 구한다.
	 * 
	 * @param key
	 * @return 해당 캐쉬 데이터의 만료시간. 해당 데이터가 없으면 0을 리턴
	 */
	public long getExpireTime(Object key);

	/**
	 * 캐쉬 데이터의 사용 횟수를 구한다.
	 * 
	 * @param key
	 * @return 캐쉬 데이터 사용 횟수
	 */
	public long getHitCount(Object key);

	/**
	 * 캐쉬에서 데이터를 제거한다. expire(Object key) 메소드를 사용할 것을 권장
	 * 
	 * @param key
	 * @return 제거된 데이터
	 * @see #expire(java.lang.Object)
	 */
	public Object remove(Object key);

	/**
	 * 캐쉬의 모든 데이터를 제거한다. expireAll() 메소드를 사용할 것을 권장
	 * 
	 * @see #expireAll()
	 */
	public void removeAll();

	/**
	 * 캐쉬의 데이터를 만료시킨다. 데이터 만료 시 데이터는 남아 있게 되며, 만료된 데이터는 get() 메소드를 통해 데이터 요청 시
	 * 제거된다.
	 * 
	 * @param key
	 * @return 만료된 데이터
	 */
	public Object expire(Object key);

	/**
	 * 캐쉬의 모든 데이터를 만료시킨다.
	 */
	public void expireAll();

	/**
	 * 캐쉬에 해당 key의 데이터가 있는지 검사
	 * 
	 * @param key
	 * @return 캐쉬에 해당 key의 데이터가 존재하면 true, 존재하지 않으면 false를 리턴
	 */
	public boolean containsKey(Object key);

	/**
	 * 캐쉬에 저장된 데이터들의 key값을 Set으로 얻는다.
	 * 
	 * @return 캐쉬에 저장된 데이터들의 key
	 */
	public Set keySet();

	/**
	 * 캐쉬에 저장된 데이터의 수를 리턴한다.
	 * 
	 * @return 캐쉬에 저장된 데이터의 수
	 */
	public int size();

}
