package justfun.common.cache;

import java.util.Hashtable;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import justfun.common.cache.comparator.CacheUseTimeComparator;

/**
 * <p>
 * LRU(Least Recently Used : 최근 최소 사용) 알고리즘을 사용한 캐쉬
 * </p>
 * <p>
 * Test Code
 * </p>
 * <code>
 * 	Cache cache = new LRUCache(2);<br>
 *		<br>
 *		cache.put( "key1", "value1", Cache.UNLIMITED );<br>
 *		Thread.sleep( 1000 );<br>
 *		System.out.println( cache );<br>
 *		<br>
 *		cache.put( "key2", "value2", Cache.UNLIMITED );<br>
 *		Thread.sleep( 1000 );<br>
 *		System.out.println( cache );<br>
 *		<br>
 *		cache.put( "key3", "value3", Cache.UNLIMITED );<br>
 *		System.out.println( cache );<br>
 *		Thread.sleep( 1000 );<br>
 *		<br>
 *		System.out.println( "----------------------------------" );<br>
 *		System.out.println( cache.get("key1") );<br>
 *		System.out.println( cache );<br>
 * </code>
 * 
 * @author 조재호
 */
public class LRUCache implements Cache {

	// public static final long UNLIMITED = -1;

	protected int capacity;

	/* 데이터 캐싱 - Hashtable은 내부적으로 동기화되므로 명시적으로 동기화할 필요가 없다. */
	protected Hashtable cache;

	public LRUCache() {
		this(10);
	}

	public LRUCache(int initialCapacity) {
		this.capacity = initialCapacity;

		cache = new Hashtable(initialCapacity);
	}

	public Object put(Object key, Object value, long expireTime) {
		/* 캐쉬의 제한 용량(갯수)을 초과한 경우, 가장 오랫동안 사용되지 않은 데이터를 제거한다. */
		if (cache.size() >= capacity) {
			// 정렬
			SortedSet s_set = new TreeSet(new CacheUseTimeComparator());
			s_set.addAll(cache.values());

			// 첫번째 캐쉬 데이터 (가장 오랫동안 사용되지 않은 데이터) 만료
			CacheData firstData = (CacheData) s_set.first();
			firstData.expire();
		}

		CacheData data = new CacheData(key, value, expireTime);
		return cache.put(key, data);
	}

	public Object get(Object key) {
		CacheData data = (CacheData) cache.get(key);

		if (data == null)
			return null;
		/* 만료일이 지난 데이터일 경우, 캐쉬에서 제거 */
		if (data.getExpireTime() != UNLIMITED && System.currentTimeMillis() > data.getExpireTime()) {
			cache.remove(data.getKey());
			return null;
		}

		return data.getData();
	}

	public long getCreatedTime(Object key) {
		CacheData data = (CacheData) cache.get(key);
		if (data == null)
			return 0;

		return data.getCreatedTime();
	}

	public long getLastAccessTime(Object key) {
		CacheData data = (CacheData) cache.get(key);
		if (data == null)
			return 0;

		return data.getLastAccessTime();
	}

	public long getExpireTime(Object key) {
		CacheData data = (CacheData) cache.get(key);
		if (data == null)
			return 0;

		return data.getExpireTime();
	}

	public long getHitCount(Object key) {
		CacheData data = (CacheData) cache.get(key);
		if (data == null)
			return -1;

		return data.getHitCount();
	}

	public Object remove(Object key) {
		return cache.remove(key);
	}

	public void removeAll() {
		Object[] keys = cache.keySet().toArray();
		for (int i = keys.length - 1; i >= 0; i--)
			cache.remove(keys[i]);
	}

	public Object expire(Object key) {
		CacheData data = (CacheData) cache.get(key);
		if (data == null)
			return null;
		data.expire();

		return data;
	}

	public void expireAll() {
		Object[] keys = cache.keySet().toArray();
		for (int i = keys.length - 1; i >= 0; i--)
			expire(keys[i]);
	}

	public boolean containsKey(Object key) {
		Object data = get(key);

		return (data == null) ? false : true;
	}

	public Set keySet() {
		return cache.keySet();
	}

	public int size() {
		return cache.size();
	}

	public String toString() {
		return cache.toString();
	}
}
