package justfun.common.util;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * 웹 개발용 유틸리티
 * 
 * @author 조재호
 */
public class HttpUtil {

	/**
	 * Query String을 Parsing 하여 파라미터를 추출한다.
	 * 
	 * @param queryString
	 * @return Request Parameter 데이터
	 */
	public static Map parseQueryString(String queryString) {
		if (queryString == null)
			return null;

		Map map = new HashMap();

		String[] params = queryString.split("[&]");
		String[] param = null;
		for (int i = params.length - 1; i >= 0; i--) {
			param = params[i].split("[=]");
			if (param.length == 2)
				map.put(param[0], param[1]);
		}

		return map;
	}

	/**
	 * Map을 QueryString으로 만든다. Map 데이터의 key와 value는 모두 String이어야 한다.
	 * 
	 * @param data
	 * @return QueryString
	 */
	public static String makeQueryString(Map data) {
		if (data == null)
			return null;

		String[] keys = (String[]) data.keySet().toArray(new String[0]);
		int keyCount = keys.length;

		StringBuffer queryString = new StringBuffer();
		for (int i = 0; i < keyCount; i++) {
			queryString.append(keys[i]).append("=").append(data.get(keys[i]));

			if (i != keyCount - 1)
				queryString.append("&");
		}

		return queryString.toString();
	}

	/**
	 * cookie_name의 key에 해당하는 쿠키 값을 얻는다.
	 * 
	 * @param request
	 * @param cookie_name
	 * @param key
	 * @return 쿠키 값
	 */
	public static String getCookie(HttpServletRequest request, String cookie_name, String key) {
		String cookie_value = getCookie(request, cookie_name);
		Map data = parseQueryString(cookie_value);

		return (String) data.get(key);
	}

	/**
	 * key-value 컬렉션을 쿠키에 저장한다.
	 * 
	 * @param response
	 * @param cookie_name
	 * @param cookie_value
	 */
	public static void setCookie(HttpServletResponse response, String cookie_name, Map cookie_value) {
		String queryString = makeQueryString(cookie_value);

		setCookie(response, cookie_name, queryString, "");
	}

	/**
	 * key-value 컬렉션을 특정 도메인으로 쿠키에 저장한다.
	 * 
	 * @param response
	 * @param cookie_name
	 * @param cookie_value
	 * @param domain
	 */
	public static void setCookie(HttpServletResponse response, String cookie_name, Map cookie_value, String domain) {
		String queryString = makeQueryString(cookie_value);

		setCookie(response, cookie_name, queryString, domain);
	}

	/**
	 * 쿠키 값을 얻는다.
	 * 
	 * @param request
	 * @param cookie_name
	 * @return 쿠키 값
	 */
	public static String getCookie(HttpServletRequest request, String cookie_name) {
		Cookie cookies[] = request.getCookies();
		Cookie an_cookie = null;
		if (cookies != null) {
			for (int i = 0; i < cookies.length; i++) {
				if (!cookies[i].getName().equals(cookie_name))
					continue;
				an_cookie = cookies[i];
				break;
			}

		}
		if (an_cookie != null)
			return an_cookie.getValue();
		else
			return null;
	}

	/**
	 * 쿠키 값을 저장한다.
	 * 
	 * @param response
	 * @param cookie_name
	 * @param cookie_value
	 */
	public static void setCookie(HttpServletResponse response, String cookie_name, String cookie_value) {
		setCookie(response, cookie_name, cookie_value, "");
	}

	/**
	 * 특정 도메인으로 쿠키 값을 저장한다.
	 * 
	 * @param response
	 * @param cookie_name
	 * @param cookie_value
	 * @param domain
	 */
	public static void setCookie(HttpServletResponse response, String cookie_name, String cookie_value, String domain) {
		try {
			Cookie cookie = new Cookie(cookie_name, cookie_value);
			if (domain != null && !domain.equals(""))
				cookie.setDomain(domain);
			cookie.setPath("/");
			response.addCookie(cookie);
		}
		catch (Exception ex) {

		}
	}

	/**
	 * 쿠키 값을 얻는다.
	 * 
	 * @param request
	 * @param cookie_name
	 * @param charset
	 * @return URLDecoding된 쿠키 값
	 */
	public static String getCookie_decode(HttpServletRequest request, String cookie_name, String charset) {
		String cookie_value = getCookie(request, cookie_name);
		try {
			if (cookie_value != null)
				cookie_value = URLDecoder.decode(cookie_value, charset);
		}
		catch (Exception ex) {
		}

		return cookie_value;
	}

	/**
	 * 특정 도메인으로 쿠키 값을 URLEncoding하여 저장한다.
	 * 
	 * @param response
	 * @param cookie_name
	 * @param cookie_value
	 * @param domain
	 * @param charset
	 */
	public static void setCookie_encode(HttpServletResponse response, String cookie_name, String cookie_value, String domain, String charset) {
		try {
			String cValue = (charset == null) ? cookie_value : URLEncoder.encode(cookie_value, charset);

			Cookie cookie = new Cookie(cookie_name, cValue);
			if (domain != null && !domain.equals(""))
				cookie.setDomain(domain);
			cookie.setPath("/");
			response.addCookie(cookie);
		}
		catch (Exception ex) {

		}
	}

	/**
	 * 쿠키를 제거한다.
	 * 
	 * @param request
	 * @param response
	 * @param cookie_name
	 */
	public static void removeCookie(HttpServletRequest request, HttpServletResponse response, String cookie_name) {
		removeCookie(request, response, cookie_name, "");
	}

	/**
	 * 특정 도메인의 쿠키를 제거한다.
	 * 
	 * @param request
	 * @param response
	 * @param cookie_name
	 * @param cookie_domain
	 */
	public static void removeCookie(HttpServletRequest request, HttpServletResponse response, String cookie_name, String cookie_domain) {
		Cookie cookies[] = request.getCookies();
		Cookie cookie = null;
		for (int i = 0; cookies != null && i < cookies.length; i++)
			if (cookies[i].getName().equals(cookie_name)) {
				cookie = cookies[i];
				cookie.setPath("/");
				if (cookie_domain != null && !cookie_domain.equals(""))
					cookie.setDomain(cookie_domain);
				cookie.setMaxAge(0);
				response.addCookie(cookie);
			}

	}

	/**
	 * cookie_name의 쿠키가 여러 키의 값을 갖고 있는지 검사
	 * 
	 * @param request
	 * @param cookie_name
	 * @return 쿠키가 여러 키의 값을 갖고 있으면 true를 리턴
	 */
	public static boolean hasKeysCookie(HttpServletRequest request, String cookie_name) {
		String cookie_value = getCookie(request, cookie_name);
		Map data = parseQueryString(cookie_value);

		return (data.size() > 1);
	}

	/**
	 * URL 호출
	 * 
	 * @param destURL
	 *            호출할 URL
	 * @param method
	 *            호출 방식 (GET, POST)
	 * @param param
	 *            파라미터 (QueryString)
	 * @return URL 호출 결과. Response Code
	 */
	public static int callURL(String destURL, String method, String param) {
		int rt = 0;

		try {
			// make parameters
			byte[] bytes = param.getBytes();

			// Create a URL pointing to the servlet or CGI script and open an
			// HttpURLConnection on that URL

			URL url = new URL(destURL);
			HttpURLConnection con = (HttpURLConnection) url.openConnection();

			// Indicate that you will be doing input and output, that the method
			// is POST, and that the content
			// length is the length of the byte array

			con.setDoOutput(true);
			con.setDoInput(true);
			con.setRequestMethod(method);
			con.setRequestProperty("Content-length", String.valueOf(bytes.length));

			// Write the parameters to the URL output stream

			OutputStream out = con.getOutputStream();
			out.write(bytes);
			out.flush();
			// Read the response
			/*
			 * BufferedReader in = new BufferedReader(new
			 * InputStreamReader(con.getInputStream())); while (true) { String
			 * line = in.readLine(); if (line == null) break;
			 * System.out.println(line); } in.close();
			 */
			out.close();
			rt = con.getResponseCode();
			con.disconnect();
		}
		catch (Exception ex) {
			// ex.printStackTrace();
			return 404;
		}

		return rt;
	}

	/**
	 * URL 호출
	 * 
	 * @param destURL
	 *            호출할 URL
	 * @param method
	 *            호출 방식 (GET, POST)
	 * @param param
	 *            파라미터 (QueryString)
	 * @return 호출한 URL의 내용
	 */
	public static String openURL(String destURL, String method, String param) {
		StringBuffer sb = new StringBuffer();

		try {
			// make parameters
			byte[] bytes = param.getBytes();

			// Create a URL pointing to the servlet or CGI script and open an
			// HttpURLConnection on that URL

			URL url = new URL(destURL);
			HttpURLConnection con = (HttpURLConnection) url.openConnection();

			// Indicate that you will be doing input and output, that the method
			// is POST, and that the content
			// length is the length of the byte array

			con.setDoOutput(true);
			con.setDoInput(true);
			con.setRequestMethod(method);
			con.setRequestProperty("Content-length", String.valueOf(bytes.length));

			// Write the parameters to the URL output stream

			OutputStream out = con.getOutputStream();
			out.write(bytes);
			out.flush();

			// Read the response
			BufferedReader in = new BufferedReader(new InputStreamReader(con.getInputStream()));
			String line;
			while (true) {
				line = in.readLine();
				if (line == null)
					break;
				sb.append(line).append("\n");
			}
			in.close();
			out.close();
			// rt = con.getResponseCode();
			con.disconnect();
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}

		return sb.toString();
	}
}
