/*
 * 데이터베이스와 인터페이스
 *
 */

package justfun.dao.helper.util;

import justfun.dao.helper.model.DatabaseTableModel;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import javax.swing.table.TableModel;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;
import justfun.dao.helper.model.ColumnNodeItem;
import justfun.dao.helper.model.DatabaseNodeItem;
import justfun.dao.helper.model.TableNodeItem;
import justfun.dao.helper.model.TreeNodeItem;

/**
 *
 * @author jhcho
 */
public class Database {
    
    public static void validateConnection(Map connectionInfo) throws Exception {
        Connection conn = makeConnection(connectionInfo);
        if( conn == null )
            throw new Exception("Can't connect to Database");
        conn.close();
    }
    
    /**
     * Connection 객체 생성
     */
    private static Connection makeConnection(Map connectionInfo) throws Exception{
        String host = connectionInfo.get("host").toString();
        int port = Integer.parseInt(connectionInfo.get("port").toString());
        String user = connectionInfo.get("user").toString();
        String password = connectionInfo.get("password").toString();
        String dbms = connectionInfo.get("dbms").toString();
        String driver = connectionInfo.get("driver").toString();
        String url = connectionInfo.get("url").toString();
        
        Class.forName(driver);
        Connection conn = DriverManager.getConnection(url, user, password);
        // conn.setReadOnly(true);
        conn.setAutoCommit(true);
        
        return conn;
    }
    
    /**
     * ResultSet을 List로 변환. (List of Map)
     */
    public static List resultSetToList(ResultSet rs) throws SQLException {
        List list = new Vector();
        
        if( rs != null ){
            ResultSetMetaData rsmd = rs.getMetaData();
            int columnCount = rsmd.getColumnCount();
            
            String[] columnNames = new String[columnCount];
            for( int i = 1; i <= columnCount; i++ ){
                columnNames[i-1] = rsmd.getColumnLabel(i);
            }
            
            Map data;
            while( rs.next() ){
                data = new HashMap();
                for( int i = 1; i <= columnCount; i++ ){
                    data.put(columnNames[i-1], rs.getObject(columnNames[i-1]));
                }
                list.add(data);
            }
            rs.close();
        }
        return list;
    }
    
    /**
     * 데이터베이스 스키마 정보를 TreeModel로 반환
     *
     * @return TreeModel
     */
    public static TreeModel getDatabaseSchema(Map connectionInfo) throws Exception {
        if( connectionInfo == null ){
            return new DefaultTreeModel(new DefaultMutableTreeNode(new TreeNodeItem("Database (not connected)")));
        }
        
        String dbms = connectionInfo.get("dbms").toString();
        Connection conn = makeConnection(connectionInfo);
        try{
            if( dbms.equalsIgnoreCase("oracle") ){
                return getOracleDatabaseSchema(conn, connectionInfo.get("name").toString());
            }
            return getDefaultDatabaseSchema(conn, connectionInfo.get("name").toString());
        }finally{
            conn.close();
        }
    }
    
    public static void attatchTableSchema(Map connectionInfo, DefaultMutableTreeNode node) throws Exception {
        String dbms = connectionInfo.get("dbms").toString();
        Connection conn = makeConnection(connectionInfo);
        try{
            if( dbms.equalsIgnoreCase("oracle") ){
                attatchOracleTableSchema(conn, node);
                return;
            }
            attatchDefaultTableSchema(conn, node);
        }finally{
            conn.close();
        }
    }
    
    private static TreeModel getDefaultDatabaseSchema(Connection conn, String connectionName) throws Exception {
        DefaultTreeModel model = new DefaultTreeModel(null);
        
        DatabaseMetaData metadata = conn.getMetaData();
        
        // make root node
        DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode(new TreeNodeItem("Database ("+ connectionName +")"));
        model.setRoot(rootNode);
        
        // catalogs (oracle tablespaces, mysql scheme...)
        List catalogs = resultSetToList(metadata.getCatalogs());
        for( int i = 0; i < catalogs.size(); i++ ){
            Map data = (Map) catalogs.get(i);
            String catalogName = data.get("TABLE_CAT").toString();
            DatabaseNodeItem item = new DatabaseNodeItem(catalogName);
            item.setConnected( catalogName.equals(conn.getCatalog()) );
            
            DefaultMutableTreeNode categoryNode = new DefaultMutableTreeNode(item);
            
            rootNode.add(categoryNode);
        }
        
        return model;
    }
    
    private static TreeModel getOracleDatabaseSchema(Connection conn, String connectionName) throws Exception {
        DefaultTreeModel model = new DefaultTreeModel(null);
        
        DatabaseMetaData metadata = conn.getMetaData();
        
        // make root node
        DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode(new TreeNodeItem("Database ("+ connectionName +")"));
        model.setRoot(rootNode);
        
        // schemas (oracle tablespaces, mysql scheme...)
        List schemas = resultSetToList(metadata.getSchemas());
        for( int i = 0; i < schemas.size(); i++ ){
            Map data = (Map) schemas.get(i);
            String schemaName = data.get("TABLE_SCHEM").toString();
            DatabaseNodeItem item = new DatabaseNodeItem(schemaName);
            item.setConnected( schemaName.equals(conn.getCatalog()) );
            
            DefaultMutableTreeNode categoryNode = new DefaultMutableTreeNode(item);
            rootNode.add(categoryNode);
        }
        
        return model;
    }
    
    private static void attatchDefaultTableSchema(Connection conn, DefaultMutableTreeNode node) throws Exception {
        
        DatabaseMetaData metadata = conn.getMetaData();
        String catalogName = node.getUserObject().toString();
        DefaultMutableTreeNode categoryNode = node;
        node.removeAllChildren();
        
        // tables of category
        List tables = resultSetToList(metadata.getTables(catalogName, "%%", "%%", null));
        Map data;
        for( int x = 0; x < tables.size(); x++ ){
            data = (Map) tables.get(x);
            String tableName = data.get("TABLE_NAME").toString();
            DefaultMutableTreeNode tableNode = new DefaultMutableTreeNode(new TableNodeItem(tableName));
            categoryNode.add(tableNode);
            
            // constraints (PK) of table
            List constraints = resultSetToList(metadata.getPrimaryKeys(catalogName, "%%", tableName));
            List primaryKeys = new ArrayList();
            for( int z = 0; z < constraints.size(); z++ ){
                data = (Map) constraints.get(z);
                primaryKeys.add(data.get("COLUMN_NAME").toString());
            }
            
            // columns of table
            List columns = resultSetToList(metadata.getColumns(catalogName, "%%", tableName, "%%"));
            for( int y = 0; y < columns.size(); y++ ){
                data = (Map) columns.get(y);
                String columnName = data.get("COLUMN_NAME").toString();
                String columnSize = data.get("COLUMN_SIZE").toString();
                String columnType = data.get("DATA_TYPE").toString();
                
                ColumnNodeItem item = new ColumnNodeItem(columnName, Integer.parseInt(columnType), Integer.parseInt(columnSize), new Boolean(primaryKeys.contains(columnName)), false);
                DefaultMutableTreeNode columnNode = new DefaultMutableTreeNode(item);
                tableNode.add(columnNode);
            }
        }
    }
    
    private static void attatchOracleTableSchema(Connection conn, DefaultMutableTreeNode node) throws Exception {
        DatabaseMetaData metadata = conn.getMetaData();
        String schemaName = node.getUserObject().toString();
        DefaultMutableTreeNode schemaNode = node;
        node.removeAllChildren();
        
        // tables of category
        List tables = resultSetToList(metadata.getTables("%%", schemaName, "%%", null));
        Map data;
        for( int x = 0; x < tables.size(); x++ ){
            data = (Map) tables.get(x);
            String tableName = data.get("TABLE_NAME").toString();
            DefaultMutableTreeNode tableNode = new DefaultMutableTreeNode(new TableNodeItem(tableName));
            schemaNode.add(tableNode);
            
            // constraints (PK) of table
            List constraints = resultSetToList(metadata.getPrimaryKeys(schemaName, "%%", tableName));
            List primaryKeys = new ArrayList();
            for( int z = 0; z < constraints.size(); z++ ){
                data = (Map) constraints.get(z);
                primaryKeys.add(data.get("COLUMN_NAME").toString());
            }
            
            // columns of table
            List columns = resultSetToList(metadata.getColumns("%%", schemaName, tableName, "%%"));
            for( int y = 0; y < columns.size(); y++ ){
                data = (Map) columns.get(y);
                String columnName = data.get("COLUMN_NAME").toString();
                String columnSize = data.get("COLUMN_SIZE").toString();
                String columnType = data.get("DATA_TYPE").toString();
                
                ColumnNodeItem item = new ColumnNodeItem(columnName, Integer.parseInt(columnType), Integer.parseInt(columnSize), new Boolean(primaryKeys.contains(columnName)), false);
                DefaultMutableTreeNode columnNode = new DefaultMutableTreeNode(item);
                tableNode.add(columnNode);
            }
        }
    }
    
    /**
     * Execute Query & return TableModel
     *
     * @return TableModel
     */
    public static TableModel executeQuery(Map connectionInfo, String catalog, String sql) throws Exception {
        DatabaseTableModel model = new DatabaseTableModel();
        
        Connection conn = makeConnection(connectionInfo);
        
        Statement stmt = null;
        try{
            if( catalog != null )
                conn.setCatalog(catalog);
            
            stmt = conn.createStatement();
            
            ResultSet rs = stmt.executeQuery(sql);
            ResultSetMetaData rsmd = rs.getMetaData();
            
            // get column
            int columnCount = rsmd.getColumnCount();
            String[] columnNames = new String[columnCount];
            int[] columnSize = new int[columnCount];
            for( int i = 1; i <= columnCount; i++ ){
                columnNames[i-1] = rsmd.getColumnLabel(i);
            }
            
            // get data
            List data = new Vector();
            Object[] record = null;
            while( rs.next() ){
                record = new Object[columnCount];
                for( int i = 1; i <= columnCount; i++ ){
                    record[i-1] = rs.getObject(i);
                }
                data.add(record);
            }
            
            model.setColumnNames(columnNames);
            model.setData(data);
            
            rs.close();
        } finally{
            try{
                if( stmt != null ) stmt.close();
            }catch(Exception ex){}
            
            conn.close();
        }
        
        return model;
    }
    
    public static int executeUpdate(Map connectionInfo, String catalog, String sql) throws Exception {
        Connection conn = makeConnection(connectionInfo);
        
        Statement stmt = null;
        try{
            if( catalog != null )
                conn.setCatalog(catalog);
            
            stmt = conn.createStatement();
            return stmt.executeUpdate(sql);
            
        } finally{
            try{
                if( stmt != null ) stmt.close();
            }catch(Exception ex){}
            
            conn.close();
        }
    }
}
