/*
 * Created on 2005. 10. 13.
 *
 */
package justfun.dao;

import java.util.ArrayList;
import java.util.List;

/**
 * DAO 객체를 사용한 일련의 프로세스(업무 흐름)을 보장하여 작업을 수행하기 위한 클래스<br>
 * <br>
 * DAOWorker는 흐름을 보장할 뿐이고 일련의 작업간의 관계 및 처리 조건에는 관여하지 않으며, 일련의 작업은 한 트랜잭션으로 처리된다.<br>
 * 따라서 다음과 같은 경우는 DAO 객체를 사용하길 권장한다.<br>
 * 1. 한 트랜잭션으로 처리되지 않는 로직인 경우<br>
 * 2. 특정 쿼리 수행 결과에 따라 로직이 달라지는 경우<br>
 * 
 * @author 조재호
 * 
 */
public class DAOWorker {

	private DAO dao;
	private List taskList;

	/**
	 * 생성자. DAOManager의 getWorker() 메소드를 통해 DAOWorker 객체를 얻는다.
	 * 
	 */
	protected DAOWorker(DAO dao) {
		this.dao = dao;
		taskList = new ArrayList();
	}

	/**
	 * 새로운 작업을 추가한다.
	 * 
	 * @param task
	 *            수행할 작업
	 */
	public void addTask(Task task) {
		if (task != null)
			taskList.add(task);
	}

	/**
	 * Worker에 등록된 작업을 수행한다. 등록된 일련의 작업들은 "순서대로", "하나의 트랜잭션으로 처리"된다.
	 * 
	 * @return 작업 수행 결과. List of Task Object
	 * @throws Exception
	 */
	public List doWork() throws Exception {

		if (dao == null) {
			throw new DAOException("dao is null");
		}

		if (taskList == null || taskList.size() == 0) {
			throw new DAOException("Worker required a minumum task of one!!");
		}

		try {
			dao.beginTransaction();

			Task task;
			for (int i = 0; i < taskList.size(); i++) {
				task = (Task) taskList.get(i);

				switch (task.getType()) {
					case Task.SELECT:
						task.setResult(dao.select(task));
						break;
					case Task.UPDATE:
						task.setResult(new Integer(dao.update(task)));
						break;
					case Task.PROCEDURE:
						dao.procedure(task);
						break;
					case Task.FUNCTION:
						task.setResult(dao.function(task));
						break;
					default:
				}
			}

			dao.commit();
		}
		catch (Exception ex) {
			dao.rollback();
			throw ex;
		}
		finally {
			dao.destroy();
		}

		return taskList;
	}
}
