package justfun.framework.svc.config;

import java.util.List;

import sun.security.krb5.Config;

/**
 * ServiceConfig 인터페이스 정의
 * 
 * @author 조재호
 * 
 */
public interface ServiceConfig {

	/**
	 * 해당 경로의 환경 설정 파일로부터 Config 정보를 load한다.
	 * 
	 * @param configName
	 * @throws Config
	 *             정보의 load에 실패하면 ServiceConfigException이 발생한다.
	 */
	public void load(String configName) throws ServiceConfigException;

	/**
	 * 설정 정보를 reload한다.
	 * 
	 * @throws ServiceConfigException
	 */
	public void reload() throws ServiceConfigException;

	/**
	 * 설정 정보 load시의 time을 구한다.
	 * 
	 * @return 설정 정보 load time
	 */
	public long getConfigTime();

	/**
	 * 현재 설정 파일의 최종 수정일을 구한다.
	 * 
	 * @return 설정 파일의 최종 수정일
	 */
	public long getLastModified();

	/**
	 * 설정 파일 수정 여부
	 * 
	 * @return 설정 파일이 수정되었으면 true, 그렇지 않으면 false를 리턴
	 */
	public boolean isModified();

	/**
	 * 설정 파일에 정의된 키들을 얻는다.
	 * 
	 * @return 설정 파일에 정의된 키들의 집합인 Set을 리턴한다.
	 */
	public List getKeys();

	/**
	 * 설정값이 존재하는지 검사
	 * 
	 * @param key
	 * @return 해당 key의 값이 존재하면 true, 존재하지 않으면 false를 리턴한다.
	 * @throws key
	 *             파라미터가 <code>null</code>이면, NullPointerException이 발생한다.
	 */
	public boolean containsKey(String key);

	/**
	 * 설정 값을 얻는다.
	 * 
	 * @return 해당 key의 값을 얻는다. 매핑되는 key가 없으면 <code>null</code>을 리턴한다.
	 * @throws key
	 *             파라미터가 <code>null</code>이면, NullPointerException이 발생한다.
	 */
	public String getValue(String key);

	/**
	 * 설정 값을 얻는다.
	 * 
	 * @param key
	 * @param defaultValue
	 * @return 해당 key의 값을 얻는다. 매핑되는 key가 없으면 <code>defaultValue</code>를 리턴한다.
	 * @throws key
	 *             파라미터가 <code>null</code>이면, NullPointerException이 발생한다.
	 */
	public String getValue(String key, String defaultValue);

	/**
	 * 서비스의 이름 설정 값을 얻는다.
	 * 
	 * @param serviceID
	 * @return serviceID에 해당하는 서비스 이름을 리턴한다.
	 * @throws serviceID
	 *             파라미터가 <code>null</code>이면, NullPointerException이 발생한다.
	 */
	public String getServiceName(String serviceID);

	/**
	 * 서비스의 타입 설정 값을 얻는다.
	 * 
	 * @param serviceID
	 * @return serviceID에 해당하는 서비스 타입을 리턴한다.
	 * @throws serviceID
	 *             파라미터가 <code>null</code>이면, NullPointerException이 발생한다.
	 */
	public String getServiceType(String serviceID);

}
