package justfun.framework.svc.factories;

import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.Map;

import justfun.framework.svc.config.ServiceConfig;
import justfun.framework.svc.factory.AbstractServiceFactory;
import justfun.framework.svc.factory.ServiceFactoryException;
import justfun.framework.svc.factory.ServiceFactoryManager;
import justfun.framework.svc.service.Service;

/**
 * 서비스 팩토리 구현. 내부적으로 서비스 객체를 캐싱한다. <strong>따라서 이 서비스 팩토리가 관리하는 서비스 객체들은 반드시 쓰레드에
 * 안전함을 보장해야 한다.</strong>
 * 
 * @author 조재호
 */
public class CachedServiceFactory extends AbstractServiceFactory {

	/* ServiceFactoryManager */
	protected ServiceFactoryManager factoryManager = ServiceFactoryManager.getInstance();

	/* for MessageResourceBundle */
	protected Class clazz = this.getClass();

	/* for Service's Constructor */
	protected Class[] type = new Class[] { String.class, String.class, ServiceConfig.class };

	/* for Service Object Cache */
	protected Map services = new HashMap();

	/**
	 * @param configName
	 * @param config
	 */
	public CachedServiceFactory(String configName, ServiceConfig config)
			throws ServiceFactoryException {
		super(configName, config);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see justfun.framework.svc.factory.ServiceFactoryIF#findService(java.lang.String)
	 */
	public Service findService(String serviceID) throws ServiceFactoryException {

		/* Service가 없으면 Exception 발생 */
		if (!config.containsKey("service." + serviceID))
			throw new ServiceFactoryException("Can't find Service : key service." + serviceID);

		/* Service Type 설정 값을 얻는다. */
		String svcType = config.getServiceType(serviceID);
		if (svcType == null)
			throw new ServiceFactoryException("Can'y find Service Type : key service." + serviceID + ".type");

		/* Service 이름을 얻는다. */
		String svcName = config.getServiceName(serviceID);

		/* Service 객체를 생성하고 캐싱한다. */
		synchronized (services) {
			Service service = (Service) services.get(serviceID);

			try {
				if (service == null) {
					Class clazz = Class.forName(svcType);
					Constructor constructor = clazz.getConstructor(type);
					Object obj = constructor.newInstance(new Object[] { serviceID, svcName, config });
					service = (Service) obj;

					services.put(serviceID, service);
				}

			}
			catch (ClassCastException cce) {
				factoryManager.removeFactory(configName);

				throw new ServiceFactoryException("Invalid Service Type : " + serviceID, cce);
			}
			catch (Exception ex) {
				factoryManager.removeFactory(configName);

				throw new ServiceFactoryException(ex);
			}

			return service;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see justfun.framework.svc.factory.ServiceFactory#getConfig()
	 */
	public ServiceConfig getConfig() {
		return config;
	}

}
