package justfun.framework.svc.factory;

import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.Map;

import justfun.framework.svc.config.PropertiesServiceConfig;
import justfun.framework.svc.config.ServiceConfig;

/**
 * 서비스 팩토리 관리자. 설정 파일로부터 서비스 팩토리 객체를 생성한다.
 * 
 * @author 조재호
 */
public final class ServiceFactoryManager {

	// protected static final String CONST_FACTORY_ID = "service.factory.id";
	protected final String CONST_FACTORY_TYPE = "service.factory.type";

	/* for ServiceFactory Constructor */
	protected Class[] type = new Class[] { String.class, ServiceConfig.class };

	private Map factories = new HashMap();

	private ServiceFactoryManager() {

	}

	/**
	 * 설정 파일의 정보로부터 ServiceFactory를 얻는다.
	 * 
	 * @param configName
	 * @return 서비스 팩토리 객체
	 * @throws 해당
	 *             설정 파일정보로 부터 서비스 팩토리 객체를 얻을 수 없을 때, Exception이 발생한다.
	 * @throws configName
	 *             인수가 null일 때 NullPointerException이 발생한다.
	 */
	public ServiceFactory findFactory(String configName) throws Exception {

		synchronized (factories) {
			/* 저장된 ServiceFactory 객체를 얻는다. */
			ServiceFactory factory = (ServiceFactory) factories.get(configName);

			if (factory == null) {
				/* 환경 설정 정보를 로드한다. */
				ServiceConfig config = new PropertiesServiceConfig();
				config.load(configName);

				/* ServiceFactory 타입 설정을 얻는다. */
				// String factoryID = (String) property.get( CONST_FACTORY_ID );
				String factoryType = (String) config.getValue(CONST_FACTORY_TYPE);

				/* 해당 타입의 ServiceFactory 객체를 생성한다. */
				Class clazz = Class.forName(factoryType);
				Constructor constructor = clazz.getConstructor(type);
				Object obj = constructor.newInstance(new Object[] { configName, config });

				/* ServiceFactory를 캐싱 */
				factory = (ServiceFactory) obj;
				factories.put(configName, factory);

			}
			else {
				/* 환경 설정 정보가 변경되었는지 체크 */
				ServiceConfig config = factory.getConfig();
				if (config.isModified()) {
					config.reload();
				}
			}

			return factory;
		}

	}

	/**
	 * ServiceFactory를 캐쉬에서 제거한다.
	 * 
	 * @param configName
	 * @throws configName
	 *             인수가 null일 때 NullPointerException이 발생한다.
	 */
	public void removeFactory(String configName) {
		synchronized (factories) {
			factories.remove(configName);
		}
	}

	/**
	 * ServiceFactoryManager의 instance를 얻는다.
	 * 
	 * @return ServiceFactoryManager의 Singleton Instance
	 */
	public static ServiceFactoryManager getInstance() {
		return ServiceFactoryManagerHolder.INSTANCE;
	}

	// 보유자 클래스
	private static class ServiceFactoryManagerHolder {
		static final ServiceFactoryManager INSTANCE = new ServiceFactoryManager();
	}

}
