/*
 * Created on 2005. 7. 4.
 *
 */
package justfun.dao;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import justfun.common.dbutil.QueryRunnerUtil;
import justfun.common.pool.Pool;

import org.apache.commons.dbutils.handlers.MapListHandler;

/**
 * DB 작업(SELECT/INSERT/UPDATE/DELETE/call procedure/call function)을 수행한다.
 * 
 * @author 조재호
 * 
 */
public class DAO {

	private Pool pool = null;
	private Connection conn = null;

	private QueryRunnerUtil runner = null;

	/**
	 * 생성자. DAOManager의 getDAO() 메소드를 통해 DAO 객체를 얻는다.
	 * 
	 * @param pool
	 */
	protected DAO(Pool pool) {
		this.pool = pool;
		conn = (Connection) pool.requestObject();
		runner = new QueryRunnerUtil();
	}

	/**
	 * DAOManager의 Pool로부터 Connection을 얻지 않고, 외부 Connection 자원을 사용할 수 있도록 한다.
	 * 
	 * @param conn
	 */
	public DAO(Connection conn) {
		this.conn = conn;
		runner = new QueryRunnerUtil();
	}

	/**
	 * SELECT 쿼리를 수행한다.
	 * 
	 * @param query
	 *            수행할 쿼리
	 * @param args
	 *            PreparedStatement의 바인딩 변수
	 * @return 쿼리 수행 결과가 List로 반환된다. List의 Element는 Map으로 구성되어 있으며, Map은 DB의 한
	 *         Record를 표현한다. Map.get("컬럼명")으로 값을 얻을 수 있다.
	 * @throws SQLException
	 */
	public List select(String query, Object[] args) throws SQLException {
		return (List) runner.query(conn, query, args, new MapListHandler());
	}

	/**
	 * SELECT 쿼리를 수행한다.
	 * 
	 * @param task
	 *            수행할 작업
	 * @return
	 * @throws SQLException
	 */
	public List select(Task task) throws SQLException {
		return select(task.getQuery(), task.getParam());
	}

	/**
	 * SELECT 쿼리를 수행하고, 특정 index의 row를 얻는다.
	 * 
	 * @param query
	 * @param args
	 * @param rowIndex
	 *            index of row (0 base) : row의 index 값. 0부터 시작한다.
	 * @return 해당 row의 데이터를 리턴한다. rowIndex가 총 row 수를 넘으면, null을 리턴한다.
	 * @throws SQLException
	 * @throws IllegalArgumentException
	 *             rowIndex가 0보다 작으면 발생한다.
	 * @version 0.2
	 */
	public Map selectRow(String query, Object[] args, int rowIndex) throws SQLException,
			IllegalArgumentException {
		if (rowIndex < 0)
			throw new IllegalArgumentException("Invalid Argument - Row index : " + rowIndex);

		List list = select(query, args);

		return (list != null && list.size() > rowIndex) ? ((Map) list.get(rowIndex)) : null;
	}

	/**
	 * SELECT 쿼리를 수행하고, 특정 index의 row를 얻는다.
	 * 
	 * @param task
	 * @param rowIndex
	 *            index of row (0 base) : row의 index 값. 0부터 시작한다.
	 * @return 해당 row의 데이터를 리턴한다. rowIndex가 총 row 수를 넘으면, null을 리턴한다.
	 * @throws SQLException
	 * @throws IllegalArgumentException
	 *             rowIndex가 0보다 작으면 발생한다.
	 * @version 0.2
	 */
	public Map selectRow(Task task, int rowIndex) throws SQLException, IllegalArgumentException {
		if (rowIndex < 0)
			throw new IllegalArgumentException("Invalid Argument - Row index : " + rowIndex);

		List list = select(task);

		return (list != null && list.size() > rowIndex) ? ((Map) list.get(rowIndex)) : null;
	}

	/**
	 * INSERT, UPDATE, DELETE 쿼리를 수행한다.
	 * 
	 * @param query
	 *            수행할 쿼리
	 * @param args
	 *            PreparedStatement의 바인딩 변수
	 * @return 쿼리 수행 결과 적용 행 수
	 * @throws SQLException
	 */
	public int update(String query, Object[] args) throws SQLException {
		return runner.update(conn, query, args);
	}

	/**
	 * INSERT, UPDATE, DELETE 쿼리를 수행한다.
	 * 
	 * @param task
	 *            수행할 작업
	 * @return
	 * @throws SQLException
	 */
	public int update(Task task) throws SQLException {
		return update(task.getQuery(), task.getParam());
	}

	/**
	 * PROCEDURE를 호출한다.
	 * 
	 * @param proc
	 *            호출 프로시져명
	 * @param args
	 *            바인딩 변수
	 * @throws SQLException
	 */
	public void procedure(String proc, Object[] args) throws SQLException {
		runner.callProcedure(conn, proc, args);
	}

	/**
	 * OutputParameter를 가진 PROCEDURE를 호출한다. return value가 존재하는 경우는 procedure보다는
	 * function을 사용할 것을 권장한다. 이 메소드가 호출하게 될 procedure의 output parameter는 반드시
	 * input parameter의 뒤에 정의되어야 한다.
	 * 
	 * ex) proc(in, in, in, out) : (O) proc(in, in, out, in) : (X)
	 * 
	 * @param proc
	 *            호출 프로시져명
	 * @param args
	 *            바인딩 변수
	 * @param outParamSqlType
	 *            output parameter의 변수 타입
	 * @return
	 * @throws SQLException
	 * @deprecated
	 */
	public List procedure(String proc, Object[] args, int[] outParamSqlType) throws SQLException {
		return runner.callProcedure(conn, proc, args, outParamSqlType);
	}

	/**
	 * PROCEDURE를 호출한다.
	 * 
	 * @param task
	 *            수행할 작업
	 * @return
	 * @throws SQLException
	 */
	public void procedure(Task task) throws SQLException {
		procedure(task.getQuery(), task.getParam());
	}

	/**
	 * FUNCTION 호출
	 * 
	 * @param func
	 *            호출 함수명
	 * @param args
	 *            바인딩 변수
	 * @return 함수의 반환값
	 * @throws SQLException
	 */
	public String function(String func, Object[] args) throws SQLException {
		return runner.callFunction(conn, func, args);
	}

	/**
	 * FUNCTION 호출
	 * 
	 * @param task
	 *            수행할 작업
	 * @return
	 * @throws SQLException
	 */
	public String function(Task task) throws SQLException {
		return function(task.getQuery(), task.getParam());
	}

	/**
	 * 트랜잭션을 시작한다. (커넥션의 AutoCommit 상태를 false로 설정한다.)
	 * 
	 * @throws DAOException
	 */
	public void beginTransaction() throws DAOException {
		try {
			conn.setAutoCommit(false);
		}
		catch (SQLException e) {
			throw new DAOException(e);
		}
	}

	/**
	 * 현재 트랜잭션을 커밋한다.
	 * 
	 * @throws DAOException
	 */
	public void commit() throws DAOException {
		try {
			conn.commit();
		}
		catch (SQLException e) {
			throw new DAOException(e);
		}
	}

	/**
	 * 현재 트랜잭션을 롤백한다.
	 * 
	 * @throws DAOException
	 */
	public void rollback() throws DAOException {
		try {
			conn.rollback();
		}
		catch (SQLException e) {
			throw new DAOException(e);
		}
	}

	/**
	 * DAO의 Destroy 작업을 수행한다. ConnectionPool에 Connection을 리턴한다. 외부 Connection을
	 * 사용한 경우 Connection을 close 한다.
	 * 
	 * @throws DAOException
	 */
	public void destroy() throws DAOException {
		try {
			if (conn != null && !conn.isClosed()) {
				if (!conn.getAutoCommit()) {
					conn.setAutoCommit(true);
				}

				if (pool != null) {
					pool.returnObject(conn);
				}
				else {
					conn.close();
				}
			}
		}
		catch (Exception ex) {
			throw new DAOException(ex);
		}
	}

	/**
	 * DAO의 Destroy 작업을 수행한다.
	 * 
	 */
	public void destroyQuietly() {
		try {
			destroy();
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}
	}
}