/*
 * Created on 2005. 6. 29.
 *
 * Pool을 이용한 Connection 자동 관리
 * DbUtils를 사용한 쿼리 실행 처리
 * 자동 트랜잭션 관리
 * 로깅
 * 캐싱
 *
 */
package justfun.dao;

import java.lang.reflect.Constructor;
import java.util.Date;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import justfun.common.pool.Pool;
import justfun.common.pool.impl.DbInfoBean;

import org.apache.commons.dbutils.QueryLoader;

/**
 * DAOManager
 * 
 * @author 조재호
 * 
 */
public class DAOManager implements DAOConstants {

	private boolean isInitialized = false;

	private Pool pool = null;
	private DbInfoBean dbinfo = null;

	private ResourceBundle resource = null;
	private Queries queries = null;

	private DAOManager() {

	}

	private static class Holder {
		static final DAOManager INSTANCE = new DAOManager();
	}

	/**
	 * DAOManager 객체를 얻는다.
	 * 
	 * @return DAOManager의 Singleton 객체
	 */
	public static DAOManager getInstance() {
		return Holder.INSTANCE;
	}

	/**
	 * DAO 객체를 얻는다.
	 * 
	 * @return DAO 객체
	 */
	public DAO getDAO() {
		DAO dao = new DAO(pool);

		return dao;
	}

	/**
	 * Queries 객체를 얻는다.
	 * 
	 * @return Queries 객체
	 */
	public Queries getQueries() {
		return queries;
	}

	/**
	 * DAOWorker 객체를 얻는다. DAOWorker를 사용하면, DAO객체를 작접 사용할 필요가 없다.
	 * 
	 * @return DAOWorker 객체
	 */
	public DAOWorker getWorker() {
		return new DAOWorker(getDAO());
	}

	/**
	 * 커넥션 풀을 생성한다.
	 * 
	 * @return 커넥션 풀 객체
	 */
	private Pool makePoolObject(DbInfoBean dbinfo) throws Exception {
		Class clazz = Class.forName(resource.getString(KEY_POOL_CLASS));
		Constructor constructor = clazz.getConstructor(new Class[] { DbInfoBean.class });
		Object obj = constructor.newInstance(new Object[] { dbinfo });

		return (Pool) obj;
	}

	/**
	 * DB 정보를 담는 DbInfoBean 객체를 생성한다.
	 * 
	 * @return
	 */
	private DbInfoBean makeDbInfoBean() {
		DbInfoBean dbinfo = new DbInfoBean();
		try {
			dbinfo.setPoolname(resource.getString(KEY_POOL_NAME));

			dbinfo.setInitconn(resource.getString(KEY_POOL_INIT_CONNECTION));
			dbinfo.setMinconn(resource.getString(KEY_POOL_MIN_CONNECTION));
			dbinfo.setMaxconn(resource.getString(KEY_POOL_MAX_CONNECTION));
			dbinfo.setConntimeout(resource.getString(KEY_POOL_CONNECTION_TIMEOUT));
			dbinfo.setReqtimeout(resource.getString(KEY_POOL_REQUEST_TIMEOUT));

			dbinfo.setDriver(resource.getString(KEY_JDBC_DRIVER));
			dbinfo.setUrl(resource.getString(KEY_JDBC_URL));
			dbinfo.setUser(resource.getString(KEY_JDBC_USER));
			dbinfo.setPw(resource.getString(KEY_JDBC_PASSWORD));
		}
		catch (MissingResourceException mre) {
			debug(mre.getMessage());
		}

		return dbinfo;
	}

	/**
	 * 쿼리 로딩 처리
	 * 
	 */
	private void initQuery() {
		queries = new Queries();

		// properties files
		try {
			if (resource.getString(KEY_QUERY_FILE_PATH) != null & !resource.getString(KEY_QUERY_FILE_PATH).equals("")) {
				String[] queryFiles = resource.getString(KEY_QUERY_FILE_PATH).split(",");

				if (queryFiles.length != 0) {
					QueryLoader loader = QueryLoader.instance();

					for (int i = queryFiles.length - 1; i >= 0; i--) {
						try {
							queries.putQueries(loader.load(queryFiles[i].trim()));
						}
						catch (Exception ex) {
							ex.printStackTrace();
						}
					}
				}
			}
		}
		catch (Exception ex) {
			debug(ex.toString());
			//ex.printStackTrace();
		}

		// xml files
		try {
			if (resource.getString(KEY_QUERY_FILE_PATH_XML) != null && !resource.getString(KEY_QUERY_FILE_PATH_XML).equals("")) {
				String[] queryFiles = resource.getString(KEY_QUERY_FILE_PATH_XML).split(",");

				if (queryFiles.length != 0) {
					XmlQueryLoader loader = XmlQueryLoader.instance();

					for (int i = queryFiles.length - 1; i >= 0; i--) {
						try {
							queries.putQueries(loader.load(queryFiles[i].trim()));
						}
						catch (Exception ex) {
							ex.printStackTrace();
						}
					}
				}
			}
		}
		catch (Exception ex) {
			debug(ex.toString());
			//ex.printStackTrace();
		}
	}

	/**
	 * 디버그 메시지 출력
	 * 
	 * @param msg
	 */
	private void debug(String msg) {
		System.out.println(msg);
	}
	
	public void printPoolLog(){
		pool.printLog(new Date().toString());
	}

	/**
	 * DAO initialize 작업 수행. Application에서 최초 한번만 수행함. 2번째 호출부터는 자동으로 무시된다.
	 * 
	 * @throws DAOException
	 */
	public void initialize() throws DAOException {
		if (!isInitialized) {
			resource = ResourceBundle.getBundle(RESOURCE_NAME);

			dbinfo = makeDbInfoBean();

			try {
				pool = makePoolObject(dbinfo);
			}
			catch (Exception ex) {
				throw new DAOException(ex);
			}

			initQuery();

			isInitialized = true;

			debug("[DAOManager]\tInitialize Done");
		}
	}

	/**
	 * DAO initialize 작업 수행. Exception 발생 시 메소드 내부에서 처리한다.
	 * 
	 * @throws DAOException
	 */
	public void initializeQuietly() {
		try {
			initialize();
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * DAO finalize 작업 수행. ConnectionPool의 Destroy 작업을 수행한다.
	 * 
	 * @throws DAOException
	 */
	public void destroy() throws DAOException {
		try {
			pool.destroy();

			debug("[DAOManager]\tDestroy Done");
		}
		catch (Exception ex) {
			throw new DAOException(ex);
		}
	}

	/**
	 * DAO finalize 작업 수행. Exception 발생 시 메소드 내부에서 처리한다.
	 * 
	 * @throws DAOException
	 */
	public void destroyQuietly() {
		try {
			destroy();
		}
		catch (Exception ex) {
			ex.printStackTrace();
		}
	}

}